/*=================================================================
 * CountPulses.c
 * 
 * This .mex file counts the number of pulses with pulse pileup
 *
 * Called by PileupStats.m -- see CountPulses subfunction
 Matlab call syntax
    function dat_out = CountPulses(dt_pulse,deadtime,t_integrate)
    inputs: 
        dt_pulse: the input inter-pulse intervals (column vector)
        deadtime: (double) the deadtime
        t_integrate: the integration time
    outputs:
        dat = [npulses kpulse];
          npulses: the number of recorded pulses
          kpulse: the number of input pulses
          returns [0 0] if insufficient number of points in dt_sample to get t_integrate total time

REA 2011-Jul-18 11:07
 COPYRIGHT:   Aprend Technology, 2011. All rights reserved.
 */
#include <math.h>
#include <float.h>
#include "mex.h"



void mexFunction( int nlhs, mxArray *plhs[], 
		  int nrhs, const mxArray*prhs[] )
     
{
	/* INPUT PARAMETERS -- see above for definitions*/
        double *dt_pulse;			/* the input inter-pulse intervals (double vector) */
	double *deadtime_pr = 0, deadtime = 0.0;	
	double *t_integrate_pr = 0, t_integrate = 0.0;	
	
	/* OUTPUT PARAMETERS */
	double *dat;		/* output data */
	
	
	/* Other variables */
	int npulses=0;
	int  kpulse = (0);
	int  npulses2gen = (0);
	double tproc = 0.0;
	double  tend = (0.0);
	
    
    /* Check for proper number of arguments */
    
    if (nrhs != 3) { 
		mexErrMsgTxt("CountPulses: three input arguments required."); 
    } else if (nlhs > 1) {
		mexErrMsgTxt("CountPulses: Too many output arguments."); 
    } 
	
    dt_pulse = mxGetPr(prhs[0]);
    deadtime_pr = mxGetPr(prhs[1]);
    deadtime = deadtime_pr[0];
    if(deadtime < FLT_MIN){
       deadtime = 1e-15;
    }
    t_integrate_pr = mxGetPr(prhs[2]);
    t_integrate = t_integrate_pr[0];
    
    /* Create a matrix for the return argument */
    plhs[0] = mxCreateDoubleMatrix(1, 2, mxREAL);
    dat = mxGetPr(plhs[0]);  /* Get pointer to the data array */
    
    // get size of input data so can test if try to overrun
    npulses2gen = mxGetM(prhs[0])*mxGetN(prhs[0]) - 1; // variable to test for overrun
	
    // do the processing
    tproc = dt_pulse[0];
    while(tproc < t_integrate){
       tend = tproc + deadtime;
       while( (kpulse<npulses2gen) & (tproc<tend) ){
          kpulse = kpulse + 1;
          tproc = tproc + dt_pulse[kpulse];
       }
       if(kpulse>=npulses2gen){
          mexErrMsgTxt("CountPulses: overran input dt_pulse array while processing");
       }
       npulses = npulses + 1;
    }	
    dat[0] = (double) npulses;
    dat[1] = (double) kpulse;
    return;
}


