function [npulses_out, n_pulses_ave, eta]  = PileupStats(varargin)
% function PileupStats
% Do a Monte Carlo simulation of pulse pileup statistics
% this version generates the number of pulses for a set of trials with fixed deadtime
%  
%  inputs:
%      mean_cps: (optional = 1e6) counts per second
%      t_integrate:  (optional = 1e-3) (seconds) the time to process pulses
%      deadtime: (optional = 1e-9) (seconds) the deadtime
%      ntrials: (optional = 100);
%  outputs: 
%      npulses_out: (vector size(ntrials,2)) the number of recorded and input pulses  for each trial
%      npulses_ave: mean_cps*t_integrate
%      eta:  deadtime*mean_cps;
%       
%  REA 2011-Jul-13 17:38 --- 2011-Oct-2 11:53
% COPYRIGHT:   Aprend Technology, 2011. All rights reserved.

% prep set of random pulses: random in time
nreqargs = 0;
assert(nargin>=nreqargs);

mean_cps = 1e6; % counts per second
     % compute pulses' start times 
t_integrate = 1e-3; % second
deadtime = 1e-9;
ntrials = 100;
if(nargin>nreqargs)
  i=1;
  while(i<=size(varargin,2))
     switch lower(varargin{i})
     case 'max';          minmax=1;
     case 'mean_cps';         mean_cps=varargin{i+1};   assert(isscalar(mean_cps));  i=i+1;
     case 't_integrate';         t_integrate=varargin{i+1};   assert(isscalar(t_integrate));  i=i+1;
     case 'deadtime';         deadtime=varargin{i+1};   assert(isscalar(deadtime));  i=i+1;
     case 'ntrials';         ntrials=varargin{i+1};   assert(isscalar(ntrials));  i=i+1;
    otherwise
      error('Unknown argument %s given',varargin{i});
     end
     i=i+1;
  end
end
eta = deadtime*mean_cps;
n_pulses_ave = mean_cps*t_integrate;
dev_npulses = sqrt(n_pulses_ave);
npulses2gen = n_pulses_ave + round(5*dev_npulses); % try to make sure have enough pulses
npulses_out = zeros(ntrials,2);
for ktrial = 1:ntrials
    dt_pulse = -(1/mean_cps)*log(rand(npulses2gen,1));% use inverse transform method to generate 
       % exponentially distributed random interpulse times
    npulses_out(ktrial,:) = CountPulses(dt_pulse,deadtime,t_integrate); % mex function  CountPulses(dt_pulse); 
          % see below for matlab code and outputs. 
end


%{
%  This code is commented out so mex function will be called. 
%  If you want to use Matlab code remove comments braces
        function dat_out = CountPulses(dt_pulse,deadtime,t_integrate)
        %  inputs: 
        %    dt_pulse: the input inter-pulse intervals
        %  outputs:
        %   dat = [npulses kpulse];
%            npulses: the number of recorded pulses
%            kpulse: the number of input pulses during the integration time
             npulses = 0; % counted pulses
             kpulse = 1; % counter into dt_pulses vector
             tproc = dt_pulse(kpulse);
             while tproc < t_integrate
                 tend = tproc + deadtime;
                 while (kpulse<= numel(dt_pulse)) && (tproc < tend)
                    kpulse = kpulse + 1;
                    tproc = tproc + dt_pulse(kpulse); 
                 end
                 npulses = npulses + 1;
             end
             dat_out = [npulses kpulse];

%}